@@ -25,6 +25,9 @@ extern void ghex_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
+/* VOID:VOID (ghex-marshal.list:4) */
+#define ghex_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID
+
 G_END_DECLS
 
 #endif /* __ghex_marshal_MARSHAL_H__ */
